# The MIT License (MIT)
#
# Copyright (c) 2015 Chris Redford
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON INFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM,OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.


# Note: commented out code on lines 52-55 and 79-82. Changes on line 122-125


from collections import OrderedDict
try:
    from collections.abc import MutableMapping, Iterable
except ImportError:
    from collections import MutableMapping, Iterable
from json import dumps
from pprint import pprint
from inspect import ismethod


class DotMap(MutableMapping, OrderedDict):
    def __init__(self, *args, **kwargs):
        self._map = OrderedDict()
        self._dynamic = kwargs.pop('_dynamic', True)
        self._prevent_method_masking = kwargs.pop('_prevent_method_masking', False)
        trackedIDs = kwargs.pop('_trackedIDs', {})

        if args:
            d = args[0]
            # for recursive assignment handling
            trackedIDs[id(d)] = self

            src = []
            if isinstance(d, MutableMapping):
                src = self.__call_items(d)
            elif isinstance(d, Iterable):
                src = d

            for k,v in src:
                # Remove this code so we can load data that has reserved key names, yet
                # still raise an exception when attempting to create them later.
                # if self._prevent_method_masking and k in reserved_keys:
                #     raise KeyError('"{}" is reserved'.format(k))
                if isinstance(v, dict):
                    idv = id(v)
                    if idv in trackedIDs:
                        v = trackedIDs[idv]
                    else:
                        trackedIDs[idv] = v
                        v = self.__class__(v, _dynamic=self._dynamic, _prevent_method_masking = self._prevent_method_masking, _trackedIDs = trackedIDs)
                if type(v) is list:
                    l = []
                    for i in v:
                        n = i
                        if isinstance(i, dict):
                            idi = id(i)
                            if idi in trackedIDs:
                                n = trackedIDs[idi]
                            else:
                                trackedIDs[idi] = i
                                n = self.__class__(i, _dynamic=self._dynamic, _prevent_method_masking = self._prevent_method_masking)
                        l.append(n)
                    v = l
                self._map[k] = v
        if kwargs:
            for k,v in self.__call_items(kwargs):
                # Remove this code so we can load data that has reserved key names, yet
                # still raise an exception when attempting to create them later.
                # if self._prevent_method_masking and k in reserved_keys:
                #     raise KeyError('"{}" is reserved'.format(k))
                self._map[k] = v

    def __call_items(self, obj):
        if hasattr(obj, 'iteritems') and ismethod(getattr(obj, 'iteritems')):
            return obj.iteritems()
        else:
            return obj.items()

    def items(self):
        return self.iteritems()

    def iteritems(self):
        return self.__call_items(self._map)

    def __iter__(self):
        return self._map.__iter__()

    def next(self):
        return self._map.next()

    def __setitem__(self, k, v):
        self._map[k] = v

    def __getitem__(self, k):
        if k not in self._map and self._dynamic and k != '_ipython_canary_method_should_not_exist_':
            # automatically extend to new DotMap
            self[k] = self.__class__()
        return self._map[k]

    def __setattr__(self, k, v):
        if k in {'_map','_dynamic', '_ipython_canary_method_should_not_exist_', '_prevent_method_masking'}:
            super(DotMap, self).__setattr__(k,v)
        elif self._prevent_method_masking and k in reserved_keys:
            raise KeyError('"{}" is reserved'.format(k))
        else:
            self[k] = v

    def __getattr__(self, k):
        if k.startswith('__') and k.endswith('__'):
            raise AttributeError(f'{k} is reserved. Please use python dict bracket notation for this key.')
        
        if self._prevent_method_masking and k in reserved_keys:
            raise AttributeError(f'{k} is reserved. Please use python dict bracket notation for this key.')

        if k in {'_map','_dynamic','_ipython_canary_method_should_not_exist_'}:
            return super(DotMap, self).__getattr__(k)

        try:
            v = super(self.__class__, self).__getattribute__(k)
            return v
        except AttributeError:
            pass

        return self[k]

    def __delattr__(self, key):
        return self._map.__delitem__(key)

    def __contains__(self, k):
        return self._map.__contains__(k)

    def __add__(self, other):
        if self.empty():
            return other
        else:
            self_type = type(self).__name__
            other_type = type(other).__name__
            msg = "unsupported operand type(s) for +: '{}' and '{}'"
            raise TypeError(msg.format(self_type, other_type))

    def __str__(self, seen = None):
        items = []
        seen = {id(self)} if seen is None else seen
        for k,v in self.__call_items(self._map):
            # circular assignment case
            if isinstance(v, self.__class__):
                if id(v) in seen:
                    items.append('{0}={1}(...)'.format(k, self.__class__.__name__))
                else:
                    seen.add(id(v))
                    items.append('{0}={1}'.format(k, v.__str__(seen)))
            else:
                items.append('{0}={1}'.format(k, repr(v)))
        joined = ', '.join(items)
        out = '{0}({1})'.format(self.__class__.__name__, joined)
        return out

    def __repr__(self):
        return str(self)

    def toDict(self, seen = None):
        if seen is None:
            seen = {}

        d = {}

        seen[id(self)] = d

        for k,v in self.items():
            if issubclass(type(v), DotMap):
                idv = id(v)
                if idv in seen:
                    v = seen[idv]
                else:
                    v = v.toDict(seen = seen)
            elif type(v) in (list, tuple):
                l = []
                for i in v:
                    n = i
                    if issubclass(type(i), DotMap):
                        idv = id(n)
                        if idv in seen:
                            n = seen[idv]
                        else:
                            n = i.toDict(seen = seen)
                    l.append(n)
                if type(v) is tuple:
                    v = tuple(l)
                else:
                    v = l
            d[k] = v
        return d

    def pprint(self, pformat='dict'):
        if pformat == 'json':
            print(dumps(self.toDict(), indent=4, sort_keys=True))
        else:
            pprint(self.toDict())

    def empty(self):
        return (not any(self))

    # proper dict subclassing
    def values(self):
        return self._map.values()

    # ipython support
    def __dir__(self):
        return self.keys()

    @classmethod
    def parseOther(self, other):
        if issubclass(type(other), DotMap):
            return other._map
        else:
            return other
    def __cmp__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__cmp__(other)
    def __eq__(self, other):
        other = DotMap.parseOther(other)
        if not isinstance(other, dict):
            return False
        return self._map.__eq__(other)
    def __ge__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__ge__(other)
    def __gt__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__gt__(other)
    def __le__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__le__(other)
    def __lt__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__lt__(other)
    def __ne__(self, other):
        other = DotMap.parseOther(other)
        return self._map.__ne__(other)

    def __delitem__(self, key):
        return self._map.__delitem__(key)
    def __len__(self):
        return self._map.__len__()
    def clear(self):
        self._map.clear()
    def copy(self):
        return self.__class__(self)
    def __copy__(self):
        return self.copy()
    def __deepcopy__(self, memo=None):
        return self.copy()
    def get(self, key, default=None):
        return self._map.get(key, default)
    def has_key(self, key):
        return key in self._map
    def iterkeys(self):
        return self._map.iterkeys()
    def itervalues(self):
        return self._map.itervalues()
    def keys(self):
        return self._map.keys()
    def pop(self, key, default=None):
        return self._map.pop(key, default)
    def popitem(self):
        return self._map.popitem()
    def setdefault(self, key, default=None):
        return self._map.setdefault(key, default)
    def update(self, *args, **kwargs):
        if len(args) != 0:
            self._map.update(*args)
        self._map.update(kwargs)
    def viewitems(self):
        return self._map.viewitems()
    def viewkeys(self):
        return self._map.viewkeys()
    def viewvalues(self):
        return self._map.viewvalues()
    @classmethod
    def fromkeys(cls, seq, value=None):
        d = cls()
        d._map = OrderedDict.fromkeys(seq, value)
        return d
    def __getstate__(self): return self.__dict__
    def __setstate__(self, d): self.__dict__.update(d)
    # bannerStr
    def _getListStr(self,items):
        out = '['
        mid = ''
        for i in items:
            mid += '  {}\n'.format(i)
        if mid != '':
            mid = '\n' + mid
        out += mid
        out += ']'
        return out
    def _getValueStr(self,k,v):
        outV = v
        multiLine = len(str(v).split('\n')) > 1
        if multiLine:
            # push to next line
            outV = '\n' + v
        if type(v) is list:
            outV = self._getListStr(v)
        out = '{} {}'.format(k,outV)
        return out
    def _getSubMapDotList(self, pre, name, subMap):
        outList = []
        if pre == '':
            pre = name
        else:
            pre = '{}.{}'.format(pre,name)
        def stamp(pre,k,v):
            valStr = self._getValueStr(k,v)
            return '{}.{}'.format(pre, valStr)
        for k,v in subMap.items():
            if isinstance(v,DotMap) and v != DotMap():
                subList = self._getSubMapDotList(pre,k,v)
                outList.extend(subList)
            else:
                outList.append(stamp(pre,k,v))
        return outList
    def _getSubMapStr(self, name, subMap):
        outList = ['== {} =='.format(name)]
        for k,v in subMap.items():
            if isinstance(v, self.__class__) and v != self.__class__():
                # break down to dots
                subList = self._getSubMapDotList('',k,v)
                # add the divit
                # subList = ['> {}'.format(i) for i in subList]
                outList.extend(subList)
            else:
                out = self._getValueStr(k,v)
                # out = '> {}'.format(out)
                out = '{}'.format(out)
                outList.append(out)
        finalOut = '\n'.join(outList)
        return finalOut
    def bannerStr(self):
        lines = []
        previous = None
        for k,v in self.items():
            if previous == self.__class__.__name__:
                lines.append('-')
            out = ''
            if isinstance(v, self.__class__):
                name = k
                subMap = v
                out = self._getSubMapStr(name,subMap)
                lines.append(out)
                previous = self.__class__.__name__
            else:
                out = self._getValueStr(k,v)
                lines.append(out)
                previous = 'other'
        lines.append('--')
        s = '\n'.join(lines)
        return s

reserved_keys = {i for i in dir(DotMap) if not i.startswith('__') and not i.endswith('__')}



"""jello - query JSON at the command line with python syntax"""

import os
import sys
import types
import ast
import json
import shutil
from keyword import iskeyword
from textwrap import TextWrapper
from jello.dotmap import DotMap

# make pygments import optional
try:
    from pygments import highlight
    from pygments.style import Style
    from pygments.token import (Name, Number, String, Keyword)
    from pygments.lexers import JsonLexer
    from pygments.lexers.javascript import JavascriptLexer
    from pygments.formatters import Terminal256Formatter
    from pygments.formatters import HtmlFormatter
    PYGMENTS_INSTALLED = True
except Exception:
    PYGMENTS_INSTALLED = False


def is_valid_variable_name(name: str) -> bool:
    dict_methods = [
        '__class__', '__class_getitem__', '__contains__', '__delattr__',
        '__delitem__', '__dir__', '__eq__', '__format__', '__ge__',
        '__getattribute__', '__getitem__', '__getstate__', '__gt__',
        '__init__', '__init_subclass__', '__ior__', '__iter__', '__le__',
        '__len__', '__lt__', '__ne__', '__new__', '__or__', '__reduce__',
        '__reduce_ex__', '__repr__', '__reversed__', '__ror__', '__setattr__',
        '__setitem__', '__sizeof__', '__str__', '__subclasshook__', 'clear',
        'copy', 'fromkeys', 'get', 'items', 'keys', 'pop', 'popitem',
        'setdefault', 'update', 'values'
    ]
    return name.isidentifier() and not iskeyword(name) and name not in dict_methods


class opts:
    initialize = None
    version_info = None
    helpme = None
    compact = None
    empty = None
    nulls = None
    raw = None
    raw_input = None
    lines = None
    force_color = None
    mono = None
    schema = None
    types = None
    keyname_color = None
    keyword_color = None
    number_color = None
    string_color = None


class JelloTheme:
    if PYGMENTS_INSTALLED:
        theme = {
            Name: 'bold ansiblue',
            Keyword: 'ansibrightblack',
            Number: 'ansimagenta',
            String: 'ansigreen'
        }

    def set_colors(self):
        """
        Updates the JelloTheme.theme dictionary used by the JelloStyle class.

        Grab custom colors from JELLO_COLORS environment variable and opts class set by .jelloconf.py file.
        Individual colors from JELLO_COLORS take precedence over .jelloconf.py. Individual colors from
        JELLO_COLORS will fall back to .jelloconf.py or default if the env variable color is set to 'default'

        JELLO_COLORS env variable takes 4 comma separated string values and should be in the format of:

        JELLO_COLORS=<keyname_color>,<keyword_color>,<number_color>,<string_color>

        Where colors are: black, red, green, yellow, blue, magenta, cyan, gray, brightblack, brightred,
                          brightgreen, brightyellow, brightblue, brightmagenta, brightcyan, white, default

        Default colors:
            JELLO_COLORS=blue,brightblack,magenta,green
            or
            JELLO_COLORS=default,default,default,default
        """
        env_colors = os.getenv('JELLO_COLORS')
        input_error = False

        if env_colors:
            color_list = env_colors.split(',')

        if env_colors and len(color_list) != 4:
            input_error = True

        if env_colors:
            for color in color_list:
                if color not in ['black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'gray', 'brightblack', 'brightred',
                                 'brightgreen', 'brightyellow', 'brightblue', 'brightmagenta', 'brightcyan', 'white', 'default']:
                    input_error = True
        else:
            color_list = ['default', 'default', 'default', 'default']

        # if there is an issue with the env variable, just set all colors to default and move on
        if input_error:
            warning_message(['could not parse JELLO_COLORS environment variable'])
            color_list = ['default', 'default', 'default', 'default']

        if PYGMENTS_INSTALLED:
            # first set theme from opts class or fallback to defaults
            self.theme = {
                Name: f'bold ansi{opts.keyname_color}' if opts.keyname_color else self.theme[Name],
                Keyword: f'ansi{opts.keyword_color}' if opts.keyword_color else self.theme[Keyword],
                Number: f'ansi{opts.number_color}' if opts.number_color else self.theme[Number],
                String: f'ansi{opts.string_color}' if opts.string_color else self.theme[String]
            }

            # then set theme from JELLO_COLORS env variable or fallback to existing colors
            self.theme = {
                Name: f'bold ansi{color_list[0]}' if not color_list[0] == 'default' else self.theme[Name],
                Keyword: f'ansi{color_list[1]}' if not color_list[1] == 'default' else self.theme[Keyword],
                Number: f'ansi{color_list[2]}' if not color_list[2] == 'default' else self.theme[Number],
                String: f'ansi{color_list[3]}' if not color_list[3] == 'default' else self.theme[String]
            }


class Schema(JelloTheme):
    """Inherits theme and set_colors() from JelloTheme"""

    def __init__(self):
        self._schema_list = []

    def color_output(self, data):
        if not opts.mono and PYGMENTS_INSTALLED:
            class JelloStyle(Style):
                styles = self.theme

            lexer = JavascriptLexer()
            formatter = Terminal256Formatter(style=JelloStyle)
            return highlight(data, lexer, formatter)[0:-1]

        else:
            return data

    def html_output(self, data):
        class JelloStyle(Style):
            styles = self.theme

        lexer = JavascriptLexer()
        formatter = HtmlFormatter(style=JelloStyle, noclasses=True)
        return highlight(data, lexer, formatter)

    def create_schema(self, data):
        self._schema_gen(data)
        myschema = '\n'.join(self._schema_list)
        # unsure if this is helpful, but trying to reduce memory footprint by clearing the list
        self._schema_list *= 0
        return myschema

    def _schema_gen(self, src, path='_'):
        """
        Creates a grep-able schema representation of the JSON.
        This method is recursive, and output is stored within self._schema_list (list).
        """
        if isinstance(src, list):
            # print empty brackets as first list definition
            val = '[]'
            val_type = ''
            padding = ''
            if opts.types:
                val_type = '//   (array)'
                padding = '  '
                if len(path) + len(val) + len(val_type) < 76:
                    padding = ' ' * (76 - (len(path) + len(val) + len(val_type)))

            self._schema_list.append(f'{path} = {val};{padding}{val_type}')

            for i, item in enumerate(src):
                self._schema_gen(item, path=f'{path}[{i}]')

        elif isinstance(src, dict):
            # print empty curly brackets as first object definition
            val = '{}'
            val_type = ''
            padding = ''
            if opts.types:
                val_type = '//  (object)'
                padding = '  '
                if len(path) + len(val) + len(val_type) < 76:
                    padding = ' ' * (76 - (len(path) + len(val) + len(val_type)))

            self._schema_list.append(f'{path} = {val};{padding}{val_type}')

            for k, v in src.items():
                # encapsulate key in brackets if it is not a valid variable name
                if is_valid_variable_name(k):
                    k = f'.{k}'
                else:
                    k = f'["{k}"]'

                if isinstance(v, list):
                    # print empty brackets as first list definition
                    val = '[]'
                    val_type = ''
                    padding = ''
                    if opts.types:
                        val_type = '//   (array)'
                        padding = '  '
                        if len(path) + len(val) + len(val_type) < 76:
                            padding = ' ' * (76 - (len(f'{path}{k}') + len(val) + len(val_type)))

                    self._schema_list.append(f'{path}{k} = {val};{padding}{val_type}')

                    for i, item in enumerate(v):
                        self._schema_gen(item, path=f'{path}{k}[{i}]')

                elif isinstance(v, dict):
                    self._schema_gen(v, path=f'{path}{k}')

                else:
                    val = json.dumps(v, ensure_ascii=False)
                    val_type = ''
                    padding = ''
                    if opts.types:
                        if val == 'true' or val == 'false':
                            val_type = '// (boolean)'
                        elif val == 'null':
                            val_type = '//    (null)'
                        elif val.replace('.', '', 1).isdigit():
                            val_type = '//  (number)'
                        else:
                            val_type = '//  (string)'

                        padding = '  '
                        if len(path) + len(k) + len(val) + len(val_type) < 76:
                            padding = ' ' * (76 - (len(path) + len(k) + len(val) + len(val_type)))

                    self._schema_list.append(f'{path}{k} = {val};{padding}{val_type}')

        else:
            val = json.dumps(src, ensure_ascii=False)
            val_type = ''
            padding = ''
            if opts.types:
                if val == 'true' or val == 'false':
                    val_type = '// (boolean)'
                elif val == 'null':
                    val_type = '//    (null)'
                elif val.replace('.', '', 1).isdigit():
                    val_type = '//  (number)'
                else:
                    val_type = '//  (string)'

                padding = '  '
                if len(path) + len(val) + len(val_type) < 76:
                    padding = ' ' * (76 - (len(path) + len(val) + len(val_type)))

            self._schema_list.append(f'{path} = {val};{padding}{val_type}')


class Json(JelloTheme):
    """Inherits theme and set_colors() from JelloTheme"""

    def color_output(self, data):
        if not opts.mono and PYGMENTS_INSTALLED:
            class JelloStyle(Style):
                styles = self.theme

            lexer = JsonLexer()
            formatter = Terminal256Formatter(style=JelloStyle)
            return highlight(data, lexer, formatter)[0:-1]

        else:
            return data

    def html_output(self, data):
        class JelloStyle(Style):
            styles = self.theme

        lexer = JsonLexer()
        formatter = HtmlFormatter(style=JelloStyle, noclasses=True)
        return highlight(data, lexer, formatter)

    def create_json(self, data):
        separators = None
        indent = 2

        if opts.compact or opts.lines:
            separators = (',', ':')
            indent = None

        if isinstance(data, dict):
            return json.dumps(data, separators=separators, indent=indent, ensure_ascii=False)

        if isinstance(data, list):
            if not opts.lines:
                return json.dumps(data, separators=separators, indent=indent, ensure_ascii=False)

            # print lines
            else:
                flat_list = ''
                for entry in data:
                    if entry is None:
                        if opts.nulls:
                            flat_list += 'null\n'
                        else:
                            flat_list += '\n'

                    elif isinstance(entry, (dict, list, bool, int, float)):
                        flat_list += json.dumps(entry, separators=separators, ensure_ascii=False) + '\n'

                    elif isinstance(entry, str):
                        # replace \n with \\n here so lines with newlines literally print the \n char
                        entry = entry.replace('\n', '\\n')
                        if opts.raw:
                            flat_list += f'{entry}' + '\n'
                        else:
                            flat_list += f'"{entry}"' + '\n'

                return flat_list.rstrip()

        # naked single item return case
        elif data is None:
            if opts.nulls:
                return 'null'
            else:
                return ''

        elif isinstance(data, (bool, int, float)):
            return json.dumps(data, ensure_ascii=False)

        elif isinstance(data, str):
            # replace \n with \\n here so lines with newlines literally print the \n char
            data = data.replace('\n', '\\n')
            if opts.raw:
                return f'{data}'
            else:
                return f'"{data}"'

        # only non-serializable types are left. Force an exception from json.dumps()
        else:
            json.dumps(data)
            # this code should not run, but just in case something slips by above
            raise TypeError(f'Object is not JSON serializable')


def load_json(data):
    try:
        json_dict = json.loads(data)
    except Exception as e:
        try:
            # if json.loads fails, try loading as json lines
            json_dict = [json.loads(i) for i in data.splitlines() if i.strip()]
        except Exception:
            # raise original JSON exception instead of JSON Lines exception
            raise e

    return json_dict


def warning_message(message_lines):
    """
    Prints warning message for non-fatal issues. The first line is prepended with
    'jello:  Warning - ' and subsequent lines are indented. Wraps text as needed
    based on the terminal width.

    Parameters:

        message:   (list) list of string lines

    Returns:

        None - just prints output to STDERR
    """
    columns = shutil.get_terminal_size().columns

    first_wrapper = TextWrapper(width=columns, subsequent_indent=' ' * 12)
    next_wrapper = TextWrapper(width=columns, initial_indent=' ' * 8,
                               subsequent_indent=' ' * 12)

    first_line = message_lines.pop(0)
    first_str = f'jello:  Warning - {first_line}'
    first_str = first_wrapper.fill(first_str)
    print(first_str, file=sys.stderr)

    for line in message_lines:
        if line == '':
            continue
        message = next_wrapper.fill(line)
        print(message, file=sys.stderr)

def read_file(file_path):
    with open(file_path, 'r') as f:
        return f.read()

def pyquery(data, query, add_to_scope=None):
    """Sets options and runs the user's query."""
    output = None

    # read data into '_' variable
    # if data is a list of dictionaries, then need to iterate through and convert all dictionaries to DotMap
    if isinstance(data, list):
        _ = [DotMap(i, _dynamic=False, _prevent_method_masking=True) if isinstance(i, dict)
             else i for i in data]

    elif isinstance(data, dict):
        _ = DotMap(data, _dynamic=False, _prevent_method_masking=True)

    else:
        _ = data

    # read initialization file to set colors, options, and user-defined functions
    jelloconf = ''
    conf_file = ''
    jcnf_dict = {}

    if opts.initialize:
        pyquery._ = _  # allows the data to be available to the initialization file

        if sys.platform.startswith('win32'):
            conf_file_dir = os.environ['APPDATA']
        else:
            conf_file_dir = os.environ["HOME"]

        try:
            conf_file = os.path.join(conf_file_dir, '.jelloconf.py')
            jelloconf = read_file(conf_file)

            # inject the data into the initialization module
            conf_prepend = 'from jello.lib import pyquery as __q__\n'
            conf_prepend += '_ = __q__._\n'
            jelloconf = conf_prepend + jelloconf

            # create and import the modified .jelloconf file as a normal module
            jcnf = types.ModuleType('jcnf')
            exec(jelloconf, jcnf.__dict__)
            jcnf_dict = {f: getattr(jcnf, f) for f in dir(jcnf) if not f.startswith('__')}

        except FileNotFoundError:
            raise FileNotFoundError(f'-i used and initialization file not found: {conf_file}')

    warn_options = False
    warn_colors = False

    for option in [opts.compact, opts.raw, opts.lines, opts.nulls, opts.force_color, opts.mono, opts.schema, opts.types]:
        if not isinstance(option, bool) and option is not None:
            opts.compact = opts.raw = opts.lines = opts.nulls = opts.force_color = opts.mono = opts.schema = opts.types = False
            warn_options = True

    for color_config in [opts.keyname_color, opts.keyword_color, opts.number_color, opts.string_color]:
        valid_colors = ['black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'gray', 'brightblack',
                        'brightred', 'brightgreen', 'brightyellow', 'brightblue', 'brightmagenta', 'brightcyan',
                        'white']

        if color_config not in valid_colors and color_config is not None:
            opts.keyname_color = opts.keyword_color = opts.number_color = opts.string_color = None
            warn_colors = True

    if warn_options:
        warning_message([
            f'Options must be set to True or False in {conf_file}',
            'Unsetting all options.'
        ])

    if warn_colors:
        valid_colors_string = ', '.join(valid_colors)
        warning_message([
            f'Colors must be set to one of: {valid_colors_string} in {conf_file}',
            'Unsetting all colors.'
        ])

    # add any functions in initialization file to the scope
    scope = {'_': _, 'os': os}
    scope.update(jcnf_dict)
    if add_to_scope is not None:
        scope.update(add_to_scope)

    # run the query
    block = ast.parse(query, mode='exec')

    if len(block.body) < 1:
        raise ValueError('No query found.')

    last = ast.Expression(block.body.pop().value)    # assumes last node is an expression
    exec(compile(block, '<string>', mode='exec'), scope)
    output = eval(compile(last, '<string>', mode='eval'), scope)

    # convert output back to normal dict
    if isinstance(output, list):
        output = [i.toDict() if isinstance(i, DotMap) else i for i in output]

    elif isinstance(output, DotMap):
        output = output.toDict()

    # if DotMap returns a bound function then we know it was a reserved attribute name
    if hasattr(output, '__self__'):
        raise ValueError('Reserved key name. Use bracket notation to access this key.')

    return output


if __name__ == '__main__':
    pass



#!/usr/bin/env python3

import unittest
import os
from jello.lib import opts, Schema


class MyTests(unittest.TestCase):
    def setUp(self):
        # initialize options
        opts.initialize = None
        opts.version_info = None
        opts.helpme = None
        opts.compact = None
        opts.nulls = None
        opts.raw = None
        opts.lines = None
        opts.mono = None
        opts.schema = None
        opts.types = None
        opts.keyname_color = None
        opts.keyword_color = None
        opts.number_color = None
        opts.string_color = None

        # initialize schema_lists
        self.schema = Schema()

        # initialize JELLO_COLORS env variable
        os.environ['JELLO_COLORS'] = 'default,default,default,default'

        # set the colors
        self.schema.set_colors()

        # create samples
        self.dict_sample = {
            'string': 'string\nwith newline\ncharacters in it',
            'true': True,
            'false': False,
            'null': None,
            'int': 42,
            'float': 3.14,
            'array': [
                'string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42,
                3.14
            ]
        }

        self.list_sample = [
            'string\nwith newline\ncharacters in it',
            True,
            False,
            None,
            42,
            3.14
        ]

        self.list_of_dicts_sample = [
            {
                'string': 'string\nwith newline\ncharacters in it',
                'true': True,
                'false': False,
                'null': None,
                'int': 42,
                'float': 3.14,
                'array': [
                    'string\nwith newline\ncharacters in it',
                    True,
                    False,
                    None,
                    42,
                    3.14
                ]
            },
            {
                'string': 'another string\nwith newline\ncharacters in it',
                'true': True,
                'false': False,
                'null': None,
                'int': 10001,
                'float': -400.45,
                'array': [
                    'string\nwith newline\ncharacters in it',
                    True,
                    False,
                    None,
                    -6000034,
                    999999.854321
                ]
            }
        ]

        self.list_of_lists_sample = [
            [
                'string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42,
                3.14
            ],
            [
                'another string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42001,
                -3.14
            ]
        ]

        self.dict_space_keys_nest_sample = {
            'string with spaces': 'string\nwith newline\ncharacters in it',
            'foo': {
                "another with spaces": {
                    "nested": {
                        "nested space": True
                    }
                }
            },
            'true': True,
            'false': False,
            'null': None,
            'int': 42,
            'float': 3.14,
            'array': [
                'string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42,
                3.14
            ]
        }

        self.deep_nest_sample = [[[[{"foo":[[[[1,2,3]]]]}]]]]

        self.reserved_or_invalid_keynames_sample = {
            "True": 1,
            "get": 1,
            "fromkeys": 1,
            "aquaero-hid-3-1cb1": 1,
            "if": 1,
            "return": 1,
            "__class__": 1
        }

    # ------------ Tests ------------

    #
    # Naked True
    #

    def test_true(self):
        """
        Test True
        """
        data_in = True
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[90mtrue\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_true_m(self):
        """
        Test True -m
        """
        data_in = True
        expected = '_ = true;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Naked False
    #

    def test_false(self):
        """
        Test False
        """
        data_in = False
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[90mfalse\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_false_m(self):
        """
        Test False -m
        """
        data_in = False
        expected = '_ = false;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Naked null
    #

    def test_null(self):
        """
        Test None
        """
        data_in = None
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[90mnull\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_null_m(self):
        """
        Test None -m
        """
        data_in = None
        expected = '_ = null;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # naked int
    #

    def test_int(self):
        """
        Test int
        """
        data_in = 42
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[35m42\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_int_m(self):
        """
        Test int -m
        """
        data_in = 42
        expected = '_ = 42;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # naked float
    #

    def test_float(self):
        """
        Test float
        """
        data_in = 3.14
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_float_m(self):
        """
        Test float -m
        """
        data_in = 3.14
        expected = '_ = 3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # naked string
    #

    def test_string(self):
        """
        Test string
        """
        data_in = '"string with\\nnewline char"'
        expected = '\x1b[34;01m_\x1b[39;00m = \x1b[32m"\\"string with\\\\nnewline char\\""\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_string_m(self):
        """
        Test string -m
        """
        data_in = '"string with\\nnewline char"'
        expected = '_ = "\\"string with\\\\nnewline char\\"";'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Naked Dict
    #

    def test_dict(self):
        """
        Test self.dict_sample
        """
        data_in = self.dict_sample
        expected = '\x1b[34;01m_\x1b[39;00m = {};\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mstring\x1b[39;00m = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mtrue\x1b[39m = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfalse\x1b[39m = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mnull\x1b[39m = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mint\x1b[39m = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfloat\x1b[39m = \x1b[35m3.14\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_dict_t(self):
        """
        Test self.dict_sample -t
        """
        opts.types = True
        data_in = self.dict_sample
        expected = '\x1b[34;01m_\x1b[39;00m = {};                                                             //  (object)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mstring\x1b[39;00m = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;                //  (string)\n\x1b[34;01m_\x1b[39;00m.\x1b[90mtrue\x1b[39m = \x1b[90mtrue\x1b[39m;                                                      // (boolean)\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfalse\x1b[39m = \x1b[90mfalse\x1b[39m;                                                    // (boolean)\n\x1b[34;01m_\x1b[39;00m.\x1b[90mnull\x1b[39m = \x1b[90mnull\x1b[39m;                                                      //    (null)\n\x1b[34;01m_\x1b[39;00m.\x1b[90mint\x1b[39m = \x1b[35m42\x1b[39m;                                                         //  (number)\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfloat\x1b[39m = \x1b[35m3.14\x1b[39m;                                                     //  (number)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m = [];                                                       //   (array)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;              //  (string)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;                                                  // (boolean)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;                                                 // (boolean)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;                                                  //    (null)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;                                                    //  (number)\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;                                                  //  (number)'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_dict_m(self):
        """
        Test self.dict_sample -m
        """
        data_in = self.dict_sample
        expected = '_ = {};\n_.string = "string\\nwith newline\\ncharacters in it";\n_.true = true;\n_.false = false;\n_.null = null;\n_.int = 42;\n_.float = 3.14;\n_.array = [];\n_.array[0] = "string\\nwith newline\\ncharacters in it";\n_.array[1] = true;\n_.array[2] = false;\n_.array[3] = null;\n_.array[4] = 42;\n_.array[5] = 3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    def test_dict_mt(self):
        """
        Test self.dict_sample -mt
        """
        opts.types = True
        data_in = self.dict_sample
        expected = '_ = {};                                                             //  (object)\n_.string = "string\\nwith newline\\ncharacters in it";                //  (string)\n_.true = true;                                                      // (boolean)\n_.false = false;                                                    // (boolean)\n_.null = null;                                                      //    (null)\n_.int = 42;                                                         //  (number)\n_.float = 3.14;                                                     //  (number)\n_.array = [];                                                       //   (array)\n_.array[0] = "string\\nwith newline\\ncharacters in it";              //  (string)\n_.array[1] = true;                                                  // (boolean)\n_.array[2] = false;                                                 // (boolean)\n_.array[3] = null;                                                  //    (null)\n_.array[4] = 42;                                                    //  (number)\n_.array[5] = 3.14;                                                  //  (number)'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # true in a list
    #

    def test_list_true(self):
        """
        Test [True]
        """
        data_in = [True]
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[90mtrue\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_true_m(self):
        """
        Test [True] -m
        """
        data_in = [True]
        expected = '_ = [];\n_[0] = true;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # false in a list
    #

    def test_list_false(self):
        """
        Test [False]
        """
        data_in = [False]
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[90mfalse\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_false_m(self):
        """
        Test [False] -m
        """
        data_in = [False]
        expected = '_ = [];\n_[0] = false;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # null in a list
    #

    def test_list_null(self):
        """
        Test [None]
        """
        data_in = [None]
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[90mnull\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_null_m(self):
        """
        Test [None] -m
        """
        data_in = [None]
        expected = '_ = [];\n_[0] = null;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Int in a list
    #

    def test_list_int(self):
        """
        Test [42]
        """
        data_in = [42]
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[35m42\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_int_m(self):
        """
        Test [42] -m
        """
        data_in = [42]
        expected = '_ = [];\n_[0] = 42;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Float in a list
    #

    def test_list_float(self):
        """
        Test [3.14]
        """
        data_in = [3.14]
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_float_m(self):
        """
        Test [3.14] -m
        """
        data_in = [3.14]
        expected = '_ = [];\n_[0] = 3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # String in a list
    #

    def test_list_str(self):
        """
        Test ['string with spaces\nand newline\ncharacters']
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string with spaces\\nand newline\\ncharacters"\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_str_m(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -m
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '_ = [];\n_[0] = "string with spaces\\nand newline\\ncharacters";'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # List with different types of elements
    #

    def test_list_sample(self):
        """
        Test self.list_sample
        """
        data_in = self.list_sample
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_sample_m(self):
        """
        Test self.list_sample -m
        """
        data_in = self.list_sample
        expected = '_ = [];\n_[0] = "string\\nwith newline\\ncharacters in it";\n_[1] = true;\n_[2] = false;\n_[3] = null;\n_[4] = 42;\n_[5] = 3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Dicts in a list
    #

    def test_list_dict(self):
        """
        Test self.list_of_dicts_sample
        """
        data_in = self.list_of_dicts_sample
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = {};\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01mstring\x1b[39;00m = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[90mtrue\x1b[39m = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[90mfalse\x1b[39m = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[90mnull\x1b[39m = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[90mint\x1b[39m = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[90mfloat\x1b[39m = \x1b[35m3.14\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m] = {};\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01mstring\x1b[39;00m = \x1b[32m"another string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[90mtrue\x1b[39m = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[90mfalse\x1b[39m = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[90mnull\x1b[39m = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[90mint\x1b[39m = \x1b[35m10001\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[90mfloat\x1b[39m = -\x1b[35m400.45\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m4\x1b[39m] = -\x1b[35m6000034\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m].\x1b[34;01marray\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m999999.854321\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_dict_m(self):
        """
        Test self.list_of_dicts_sample -m
        """
        data_in = self.list_of_dicts_sample
        expected = '_ = [];\n_[0] = {};\n_[0].string = "string\\nwith newline\\ncharacters in it";\n_[0].true = true;\n_[0].false = false;\n_[0].null = null;\n_[0].int = 42;\n_[0].float = 3.14;\n_[0].array = [];\n_[0].array[0] = "string\\nwith newline\\ncharacters in it";\n_[0].array[1] = true;\n_[0].array[2] = false;\n_[0].array[3] = null;\n_[0].array[4] = 42;\n_[0].array[5] = 3.14;\n_[1] = {};\n_[1].string = "another string\\nwith newline\\ncharacters in it";\n_[1].true = true;\n_[1].false = false;\n_[1].null = null;\n_[1].int = 10001;\n_[1].float = -400.45;\n_[1].array = [];\n_[1].array[0] = "string\\nwith newline\\ncharacters in it";\n_[1].array[1] = true;\n_[1].array[2] = false;\n_[1].array[3] = null;\n_[1].array[4] = -6000034;\n_[1].array[5] = 999999.854321;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # lists in list
    #

    def test_list_list(self):
        """
        Test self.list_of_lists_sample
        """
        data_in = self.list_of_lists_sample
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m0\x1b[39m] = \x1b[32m"another string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m4\x1b[39m] = \x1b[35m42001\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m1\x1b[39m][\x1b[35m5\x1b[39m] = -\x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_list_list_m(self):
        """
        Test self.list_of_lists_sample -m
        """
        data_in = self.list_of_lists_sample
        expected = '_ = [];\n_[0] = [];\n_[0][0] = "string\\nwith newline\\ncharacters in it";\n_[0][1] = true;\n_[0][2] = false;\n_[0][3] = null;\n_[0][4] = 42;\n_[0][5] = 3.14;\n_[1] = [];\n_[1][0] = "another string\\nwith newline\\ncharacters in it";\n_[1][1] = true;\n_[1][2] = false;\n_[1][3] = null;\n_[1][4] = 42001;\n_[1][5] = -3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # deep nest
    #

    def test_dict_space_keys_nest(self):
        """
        Test self.dict_space_keys_nest_sample
        """
        data_in = self.dict_space_keys_nest_sample
        expected = '\x1b[34;01m_\x1b[39;00m = {};\n\x1b[34;01m_\x1b[39;00m[\x1b[32m"string with spaces"\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mfoo\x1b[39;00m = {};\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mfoo\x1b[39;00m[\x1b[32m"another with spaces"\x1b[39m] = {};\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mfoo\x1b[39;00m[\x1b[32m"another with spaces"\x1b[39m].\x1b[34;01mnested\x1b[39;00m = {};\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01mfoo\x1b[39;00m[\x1b[32m"another with spaces"\x1b[39m].\x1b[34;01mnested\x1b[39;00m[\x1b[32m"nested space"\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mtrue\x1b[39m = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfalse\x1b[39m = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mnull\x1b[39m = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mint\x1b[39m = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[90mfloat\x1b[39m = \x1b[35m3.14\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m0\x1b[39m] = \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m1\x1b[39m] = \x1b[90mtrue\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m2\x1b[39m] = \x1b[90mfalse\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m3\x1b[39m] = \x1b[90mnull\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m4\x1b[39m] = \x1b[35m42\x1b[39m;\n\x1b[34;01m_\x1b[39;00m.\x1b[34;01marray\x1b[39;00m[\x1b[35m5\x1b[39m] = \x1b[35m3.14\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_dict_space_keys_nest_m(self):
        """
        Test self.dict_space_keys_nest_sample -m
        """
        data_in = self.dict_space_keys_nest_sample
        expected = '_ = {};\n_["string with spaces"] = "string\\nwith newline\\ncharacters in it";\n_.foo = {};\n_.foo["another with spaces"] = {};\n_.foo["another with spaces"].nested = {};\n_.foo["another with spaces"].nested["nested space"] = true;\n_.true = true;\n_.false = false;\n_.null = null;\n_.int = 42;\n_.float = 3.14;\n_.array = [];\n_.array[0] = "string\\nwith newline\\ncharacters in it";\n_.array[1] = true;\n_.array[2] = false;\n_.array[3] = null;\n_.array[4] = 42;\n_.array[5] = 3.14;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    def test_deep_nest(self):
        """
        Test self.deep_nest_sample
        """
        data_in = self.deep_nest_sample
        expected = '\x1b[34;01m_\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = {};\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = [];\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m] = \x1b[35m1\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m1\x1b[39m] = \x1b[35m2\x1b[39m;\n\x1b[34;01m_\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m].\x1b[34;01mfoo\x1b[39;00m[\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m0\x1b[39m][\x1b[35m2\x1b[39m] = \x1b[35m3\x1b[39m;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(self.schema.color_output(output), expected)

    def test_deep_nest_m(self):
        """
        Test self.deep_nest_sample -m
        """
        data_in = self.deep_nest_sample
        expected = '_ = [];\n_[0] = [];\n_[0][0] = [];\n_[0][0][0] = [];\n_[0][0][0][0] = {};\n_[0][0][0][0].foo = [];\n_[0][0][0][0].foo[0] = [];\n_[0][0][0][0].foo[0][0] = [];\n_[0][0][0][0].foo[0][0][0] = [];\n_[0][0][0][0].foo[0][0][0][0] = 1;\n_[0][0][0][0].foo[0][0][0][1] = 2;\n_[0][0][0][0].foo[0][0][0][2] = 3;'
        self.assertEqual(self.schema.create_schema(data_in), expected)

    #
    # Handle invalid or reserved key names
    #

    def test_dict_reserved_or_invalid_keynames(self):
        """
        Test self.reserved_or_invalid_keynames_sample
        """
        data_in = self.reserved_or_invalid_keynames_sample
        expected = '_ = {};\n_["True"] = 1;\n_["get"] = 1;\n_["fromkeys"] = 1;\n_["aquaero-hid-3-1cb1"] = 1;\n_["if"] = 1;\n_["return"] = 1;\n_["__class__"] = 1;'
        output = self.schema.create_schema(data_in)
        self.assertEqual(output, expected)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

import unittest
from collections import OrderedDict
import pygments
from jello.lib import opts, Json


class MyTests(unittest.TestCase):
    def setUp(self):
        # initialize options
        opts.initialize = None
        opts.version_info = None
        opts.helpme = None
        opts.compact = None
        opts.nulls = None
        opts.raw = None
        opts.lines = None
        opts.mono = None
        opts.schema = None
        opts.types = None
        opts.keyname_color = None
        opts.keyword_color = None
        opts.number_color = None
        opts.string_color = None

        # initialize Json class
        self.json_out = Json()

        # create samples
        self.dict_sample = {
            'string': 'string\nwith newline\ncharacters in it',
            'true': True,
            'false': False,
            'null': None,
            'int': 42,
            'float': 3.14,
            'array': [
                'string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42,
                3.14
            ]
        }

        self.list_sample = [
            'string\nwith newline\ncharacters in it',
            True,
            False,
            None,
            42,
            3.14
        ]

        self.list_of_dicts_sample = [
            {
                'string': 'string\nwith newline\ncharacters in it',
                'true': True,
                'false': False,
                'null': None,
                'int': 42,
                'float': 3.14,
                'array': [
                    'string\nwith newline\ncharacters in it',
                    True,
                    False,
                    None,
                    42,
                    3.14
                ]
            },
            {
                'string': 'another string\nwith newline\ncharacters in it',
                'true': True,
                'false': False,
                'null': None,
                'int': 10001,
                'float': -400.45,
                'array': [
                    'string\nwith newline\ncharacters in it',
                    True,
                    False,
                    None,
                    -6000034,
                    999999.854321
                ]
            }
        ]

        self.list_of_lists_sample = [
            [
                'string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42,
                3.14
            ],
            [
                'another string\nwith newline\ncharacters in it',
                True,
                False,
                None,
                42001,
                -3.14
            ]
        ]

    # ------------ Tests ------------

    #
    # Naked True
    #

    def test_true(self):
        """
        Test True
        """
        data_in = True
        expected = 'true'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_true_r(self):
        """
        Test True -r
        """
        data_in = True
        expected = 'true'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_true_l(self):
        """
        Test True -l
        """
        data_in = True
        expected = 'true'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_true_rl(self):
        """
        Test True -rl
        """
        data_in = True
        expected = 'true'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Naked False
    #

    def test_false(self):
        """
        Test False
        """
        data_in = False
        expected = 'false'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_false_r(self):
        """
        Test False -r
        """
        data_in = False
        expected = 'false'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_false_l(self):
        """
        Test False -l
        """
        data_in = False
        expected = 'false'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_false_rl(self):
        """
        Test False -rl
        """
        data_in = False
        expected = 'false'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Naked null
    #

    def test_null(self):
        """
        Test None
        """
        data_in = None
        expected = ''
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_null_n(self):
        """
        Test None with -n
        """
        data_in = None
        expected = 'null'
        opts.nulls = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_null_r(self):
        """
        Test None with -r
        """
        data_in = None
        expected = ''
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_null_rl(self):
        """
        Test None with -rl
        """
        data_in = None
        expected = ''
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_null_rln(self):
        """
        Test None with -rln
        """
        data_in = None
        expected = 'null'
        opts.raw = True
        opts.lines = True
        opts.nulls = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # naked int
    #

    def test_int(self):
        """
        Test int
        """
        data_in = 42
        expected = '42'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_int_r(self):
        """
        Test int -r
        """
        data_in = 42
        expected = '42'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_int_l(self):
        """
        Test int -l
        """
        data_in = 42
        expected = '42'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_int_rl(self):
        """
        Test int -rl
        """
        data_in = 42
        expected = '42'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # naked float
    #

    def test_float(self):
        """
        Test float
        """
        data_in = 3.14
        expected = '3.14'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_float_r(self):
        """
        Test float -r
        """
        data_in = 3.14
        expected = '3.14'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_float_l(self):
        """
        Test float -l
        """
        data_in = 3.14
        expected = '3.14'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_float_rl(self):
        """
        Test float -rl
        """
        data_in = 3.14
        expected = '3.14'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # naked string
    #

    def test_string(self):
        """
        Test "string with\nnewline char"
        """
        data_in = '"string with\nnewline char"'
        expected = '""string with\\nnewline char""'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_string_r(self):
        """
        Test "string with\nnewline char" -r
        """
        data_in = '"string with\nnewline char"'
        expected = '"string with\\nnewline char"'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_string_l(self):
        """
        Test "string with\nnewline char" -l
        """
        data_in = '"string with\nnewline char"'
        expected = '""string with\\nnewline char""'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_string_rl(self):
        """
        Test "string with\nnewline char" -rl
        """
        data_in = '"string with\nnewline char"'
        expected = '"string with\\nnewline char"'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Naked Dict
    #

    def test_dict(self):
        """
        Test self.dict_sample
        """
        data_in = self.dict_sample
        expected = '{\n  "string": "string\\nwith newline\\ncharacters in it",\n  "true": true,\n  "false": false,\n  "null": null,\n  "int": 42,\n  "float": 3.14,\n  "array": [\n    "string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42,\n    3.14\n  ]\n}'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_r(self):
        """
        Test self.dict_sample -r
        """
        data_in = self.dict_sample
        expected = '{\n  "string": "string\\nwith newline\\ncharacters in it",\n  "true": true,\n  "false": false,\n  "null": null,\n  "int": 42,\n  "float": 3.14,\n  "array": [\n    "string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42,\n    3.14\n  ]\n}'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_l(self):
        """
        Test self.dict_sample -l
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_c(self):
        """
        Test self.dict_sample -c
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_rl(self):
        """
        Test self.dict_sample -rl
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_cl(self):
        """
        Test self.dict_sample -cl
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_cr(self):
        """
        Test self.dict_sample -cr
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_dict_crl(self):
        """
        Test self.dict_sample -crl
        """
        data_in = self.dict_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    # only run this test if using pygments 2.9.0
    @unittest.skipIf(pygments.__version__ != '2.9.0', "Skip if Pygments 2.9.0 is not installed")
    def test_dict_html(self):
        """
        Test self.dict_sample html output
        """
        data_in = self.dict_sample
        expected = '<div class="highlight" style="background: #ffffff"><pre style="line-height: 125%;"><span></span>{\n  <span style="color: #00007f; font-weight: bold">&quot;string&quot;</span>: <span style="color: #007f00">&quot;string\\nwith newline\\ncharacters in it&quot;</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;true&quot;</span>: <span style="color: #555555">true</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;false&quot;</span>: <span style="color: #555555">false</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;null&quot;</span>: <span style="color: #555555">null</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;int&quot;</span>: <span style="color: #7f007f">42</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;float&quot;</span>: <span style="color: #7f007f">3.14</span>,\n  <span style="color: #00007f; font-weight: bold">&quot;array&quot;</span>: [\n    <span style="color: #007f00">&quot;string\\nwith newline\\ncharacters in it&quot;</span>,\n    <span style="color: #555555">true</span>,\n    <span style="color: #555555">false</span>,\n    <span style="color: #555555">null</span>,\n    <span style="color: #7f007f">42</span>,\n    <span style="color: #7f007f">3.14</span>\n  ]\n}\n</pre></div>\n'
        output = self.json_out.create_json(data_in)
        self.assertEqual(self.json_out.html_output(output), expected)

    def test_dict_color(self):
        """
        Test self.dict_sample color output
        """
        data_in = self.dict_sample
        expected = '{\n  \x1b[34;01m"string"\x1b[39;00m: \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m,\n  \x1b[34;01m"true"\x1b[39;00m: \x1b[90mtrue\x1b[39m,\n  \x1b[34;01m"false"\x1b[39;00m: \x1b[90mfalse\x1b[39m,\n  \x1b[34;01m"null"\x1b[39;00m: \x1b[90mnull\x1b[39m,\n  \x1b[34;01m"int"\x1b[39;00m: \x1b[35m42\x1b[39m,\n  \x1b[34;01m"float"\x1b[39;00m: \x1b[35m3.14\x1b[39m,\n  \x1b[34;01m"array"\x1b[39;00m: [\n    \x1b[32m"string\\nwith newline\\ncharacters in it"\x1b[39m,\n    \x1b[90mtrue\x1b[39m,\n    \x1b[90mfalse\x1b[39m,\n    \x1b[90mnull\x1b[39m,\n    \x1b[35m42\x1b[39m,\n    \x1b[35m3.14\x1b[39m\n  ]\n}'
        output = self.json_out.create_json(data_in)
        self.assertEqual(self.json_out.color_output(output), expected)

    #
    # true in a list
    #

    def test_list_true(self):
        """
        Test [True]
        """
        data_in = [True]
        expected = '[\n  true\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_c(self):
        """
        Test [True] -c
        """
        data_in = [True]
        expected = '[true]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_r(self):
        """
        Test [True] -r
        """
        data_in = [True]
        expected = '[\n  true\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_l(self):
        """
        Test [True] -l
        """
        data_in = [True]
        expected = 'true'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_cl(self):
        """
        Test [True] -cl
        """
        data_in = [True]
        expected = 'true'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_rl(self):
        """
        Test [True] -rl
        """
        data_in = [True]
        expected = 'true'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_cr(self):
        """
        Test [True] -cr
        """
        data_in = [True]
        expected = '[true]'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_true_crl(self):
        """
        Test [True] -crl
        """
        data_in = [True]
        expected = 'true'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # false in a list
    #

    def test_list_false(self):
        """
        Test [False]
        """
        data_in = [False]
        expected = '[\n  false\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_c(self):
        """
        Test [False] -c
        """
        data_in = [False]
        expected = '[false]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_r(self):
        """
        Test [False] -r
        """
        data_in = [False]
        expected = '[\n  false\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_l(self):
        """
        Test [False] -l
        """
        data_in = [False]
        expected = 'false'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_cl(self):
        """
        Test [False] -cl
        """
        data_in = [False]
        expected = 'false'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_rl(self):
        """
        Test [False] -rl
        """
        data_in = [False]
        expected = 'false'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_cr(self):
        """
        Test [False] -cr
        """
        data_in = [False]
        expected = '[false]'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_false_crl(self):
        """
        Test [False] -crl
        """
        data_in = [False]
        expected = 'false'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # null in a list
    #

    def test_list_null(self):
        """
        Test [None]
        """
        data_in = [None]
        expected = '[\n  null\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_c(self):
        """
        Test [None] -c
        """
        data_in = [None]
        expected = '[null]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_r(self):
        """
        Test [None] -r
        """
        data_in = [None]
        expected = '[\n  null\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_l(self):
        """
        Test [None] -l
        """
        data_in = [None]
        expected = ''
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_cl(self):
        """
        Test [None] -cl
        """
        data_in = [None]
        expected = ''
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_rl(self):
        """
        Test [None] -rl
        """
        data_in = [None]
        expected = ''
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_cr(self):
        """
        Test [None] -cr
        """
        data_in = [None]
        expected = '[null]'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_crl(self):
        """
        Test [False] -crl
        """
        data_in = [None]
        expected = ''
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_n(self):
        """
        Test [None] -n
        """
        data_in = [None]
        expected = '[\n  null\n]'
        opts.nulls = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_nc(self):
        """
        Test [None] -nc
        """
        data_in = [None]
        expected = '[null]'
        opts.nulls = True
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_nl(self):
        """
        Test [None] -nl
        """
        data_in = [None]
        expected = 'null'
        opts.nulls = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_nr(self):
        """
        Test [None] -nr
        """
        data_in = [None]
        expected = '[\n  null\n]'
        opts.nulls = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_ncr(self):
        """
        Test [None] -ncr
        """
        data_in = [None]
        expected = '[null]'
        opts.nulls = True
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_ncl(self):
        """
        Test [None] -ncl
        """
        data_in = [None]
        expected = 'null'
        opts.nulls = True
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_nlr(self):
        """
        Test [None] -nlr
        """
        data_in = [None]
        expected = 'null'
        opts.nulls = True
        opts.lines = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_null_nlrc(self):
        """
        Test [None] -nlrc
        """
        data_in = [None]
        expected = 'null'
        opts.nulls = True
        opts.lines = True
        opts.raw = True
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Int in a list
    #

    def test_list_int(self):
        """
        Test [integer]
        """
        data_in = [42]
        expected = '[\n  42\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_c(self):
        """
        Test [integer] -c
        """
        data_in = [42]
        expected = '[42]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_l(self):
        """
        Test [integer] -l
        """
        data_in = [42]
        expected = '42'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_r(self):
        """
        Test [integer] -r
        """
        data_in = [42]
        expected = '[\n  42\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_rl(self):
        """
        Test [integer] -rl
        """
        data_in = [42]
        expected = '42'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_cl(self):
        """
        Test [integer] -cl
        """
        data_in = [42]
        expected = '42'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_int_crl(self):
        """
        Test [integer] -crl
        """
        data_in = [42]
        expected = '42'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Float in a list
    #

    def test_list_float(self):
        """
        Test [float]
        """
        data_in = [3.14]
        expected = '[\n  3.14\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_c(self):
        """
        Test [float] -c
        """
        data_in = [3.14]
        expected = '[3.14]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_l(self):
        """
        Test [float] -l
        """
        data_in = [3.14]
        expected = '3.14'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_r(self):
        """
        Test [float] -r
        """
        data_in = [3.14]
        expected = '[\n  3.14\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_rl(self):
        """
        Test [float] -rl
        """
        data_in = [3.14]
        expected = '3.14'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_rc(self):
        """
        Test [float] -rc
        """
        data_in = [3.14]
        expected = '[3.14]'
        opts.raw = True
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_float_rcl(self):
        """
        Test [float] -rcl
        """
        data_in = [3.14]
        expected = '3.14'
        opts.raw = True
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # String in a list
    #

    def test_list_str(self):
        """
        Test ['string with spaces\nand newline\ncharacters']
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '[\n  "string with spaces\\nand newline\\ncharacters"\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_l(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -l
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '"string with spaces\\nand newline\\ncharacters"'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_r(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -r
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '[\n  "string with spaces\\nand newline\\ncharacters"\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_c(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -c
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '["string with spaces\\nand newline\\ncharacters"]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_rl(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -rl
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = 'string with spaces\\nand newline\\ncharacters'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_rc(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -rc
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '["string with spaces\\nand newline\\ncharacters"]'
        opts.raw = True
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_cl(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -cl
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = '"string with spaces\\nand newline\\ncharacters"'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_str_crl(self):
        """
        Test ['string with spaces\nand newline\ncharacters'] -crl
        """
        data_in = ['string with spaces\nand newline\ncharacters']
        expected = 'string with spaces\\nand newline\\ncharacters'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # List with different types of elements
    #

    def test_list_sample(self):
        """
        Test self.list_sample
        """
        data_in = self.list_sample
        expected = '[\n  "string\\nwith newline\\ncharacters in it",\n  true,\n  false,\n  null,\n  42,\n  3.14\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_l(self):
        """
        Test self.list_sample -l
        """
        data_in = self.list_sample
        expected = '"string\\nwith newline\\ncharacters in it"\ntrue\nfalse\n\n42\n3.14'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_r(self):
        """
        Test self.list_sample -r
        """
        data_in = self.list_sample
        expected = '[\n  "string\\nwith newline\\ncharacters in it",\n  true,\n  false,\n  null,\n  42,\n  3.14\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_c(self):
        """
        Test self.list_sample -c
        """
        data_in = self.list_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_rl(self):
        """
        Test self.list_sample -rl
        """
        data_in = self.list_sample
        expected = 'string\\nwith newline\\ncharacters in it\ntrue\nfalse\n\n42\n3.14'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_rc(self):
        """
        Test self.list_sample -rc
        """
        data_in = self.list_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]'
        opts.raw = True
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_cl(self):
        """
        Test self.list_sample -cl
        """
        data_in = self.list_sample
        expected = '"string\\nwith newline\\ncharacters in it"\ntrue\nfalse\n\n42\n3.14'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_sample_crl(self):
        """
        Test self.list_sample -crl
        """
        data_in = self.list_sample
        expected = 'string\\nwith newline\\ncharacters in it\ntrue\nfalse\n\n42\n3.14'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # Dicts in a list
    #

    def test_list_dict(self):
        """
        Test self.list_of_dicts_sample
        """
        data_in = self.list_of_dicts_sample
        expected = '[\n  {\n    "string": "string\\nwith newline\\ncharacters in it",\n    "true": true,\n    "false": false,\n    "null": null,\n    "int": 42,\n    "float": 3.14,\n    "array": [\n      "string\\nwith newline\\ncharacters in it",\n      true,\n      false,\n      null,\n      42,\n      3.14\n    ]\n  },\n  {\n    "string": "another string\\nwith newline\\ncharacters in it",\n    "true": true,\n    "false": false,\n    "null": null,\n    "int": 10001,\n    "float": -400.45,\n    "array": [\n      "string\\nwith newline\\ncharacters in it",\n      true,\n      false,\n      null,\n      -6000034,\n      999999.854321\n    ]\n  }\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_c(self):
        """
        Test self.list_of_dicts_sample -c
        """
        data_in = self.list_of_dicts_sample
        expected = '[{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]},{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_r(self):
        """
        Test self.list_of_dicts_sample -r
        """
        data_in = self.list_of_dicts_sample
        expected = '[\n  {\n    "string": "string\\nwith newline\\ncharacters in it",\n    "true": true,\n    "false": false,\n    "null": null,\n    "int": 42,\n    "float": 3.14,\n    "array": [\n      "string\\nwith newline\\ncharacters in it",\n      true,\n      false,\n      null,\n      42,\n      3.14\n    ]\n  },\n  {\n    "string": "another string\\nwith newline\\ncharacters in it",\n    "true": true,\n    "false": false,\n    "null": null,\n    "int": 10001,\n    "float": -400.45,\n    "array": [\n      "string\\nwith newline\\ncharacters in it",\n      true,\n      false,\n      null,\n      -6000034,\n      999999.854321\n    ]\n  }\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_l(self):
        """
        Test self.list_of_dicts_sample -l
        """
        data_in = self.list_of_dicts_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}\n{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_cr(self):
        """
        Test self.list_of_dicts_sample -cr
        """
        data_in = self.list_of_dicts_sample
        expected = '[{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]},{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}]'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_cl(self):
        """
        Test self.list_of_dicts_sample -cl
        """
        data_in = self.list_of_dicts_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}\n{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_rl(self):
        """
        Test self.list_of_dicts_sample -rl
        """
        data_in = self.list_of_dicts_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}\n{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_dict_crl(self):
        """
        Test self.list_of_dicts_sample -crl
        """
        data_in = self.list_of_dicts_sample
        expected = '{"string":"string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":42,"float":3.14,"array":["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]}\n{"string":"another string\\nwith newline\\ncharacters in it","true":true,"false":false,"null":null,"int":10001,"float":-400.45,"array":["string\\nwith newline\\ncharacters in it",true,false,null,-6000034,999999.854321]}'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    #
    # lists in list
    #

    def test_list_list(self):
        """
        Test self.list_of_lists_sample
        """
        data_in = self.list_of_lists_sample
        expected = '[\n  [\n    "string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42,\n    3.14\n  ],\n  [\n    "another string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42001,\n    -3.14\n  ]\n]'
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_c(self):
        """
        Test self.list_of_lists_sample -c
        """
        data_in = self.list_of_lists_sample
        expected = '[["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14],["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]]'
        opts.compact = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_r(self):
        """
        Test self.list_of_lists_sample -r
        """
        data_in = self.list_of_lists_sample
        expected = '[\n  [\n    "string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42,\n    3.14\n  ],\n  [\n    "another string\\nwith newline\\ncharacters in it",\n    true,\n    false,\n    null,\n    42001,\n    -3.14\n  ]\n]'
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_l(self):
        """
        Test self.list_of_lists_sample -l
        """
        data_in = self.list_of_lists_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]\n["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]'
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_cr(self):
        """
        Test self.list_of_lists_sample -cr
        """
        data_in = self.list_of_lists_sample
        expected = '[["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14],["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]]'
        opts.compact = True
        opts.raw = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_cl(self):
        """
        Test self.list_of_lists_sample -cl
        """
        data_in = self.list_of_lists_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]\n["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]'
        opts.compact = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_rl(self):
        """
        Test self.list_of_lists_sample -rl
        """
        data_in = self.list_of_lists_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]\n["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]'
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_list_list_crl(self):
        """
        Test self.list_of_lists_sample -crl
        """
        data_in = self.list_of_lists_sample
        expected = '["string\\nwith newline\\ncharacters in it",true,false,null,42,3.14]\n["another string\\nwith newline\\ncharacters in it",true,false,null,42001,-3.14]'
        opts.compact = True
        opts.raw = True
        opts.lines = True
        self.assertEqual(self.json_out.create_json(data_in), expected)

    def test_non_serializable(self):
        """
        Test _.items()
        """
        data_in = OrderedDict(foo='bar').items()
        self.assertRaises(TypeError, self.json_out.create_json, data_in)

    def test_non_serializable_l(self):
        """
        Test _.items() -l
        """
        data_in = OrderedDict(foo='bar').items()
        opts.lines = True
        self.assertRaises(TypeError, self.json_out.create_json, data_in)


if __name__ == '__main__':
    unittest.main()



"""jello - query JSON at the command line with python syntax"""


__version__ = '1.6.1'
AUTHOR = 'Kelly Brazil'
WEBSITE = 'https://github.com/kellyjonbrazil/jello'
COPYRIGHT = '© 2020-2025 Kelly Brazil'
LICENSE = 'MIT License'
